function out = CL_model(t,x,u,p)
% Description

%% Extracting (filtered) inputs
Nf    = x( 1);
XV1f  = x( 2);
FCP3f = x( 3);

%% Extracting states:

% Differential
H1   = x( 4);                                 % Enthalpy LP evaporator [kJ]
H2   = x( 5);                                 % Enthalpy IP evaporator [kJ]
T3   = x( 6);                                 % Temperature HP receiver [K]
W1   = x( 7);                              % Ref. holdup LP evaporator [kg]
W2   = x( 8);                              % Ref. holdup IP evaporator [kg]
W4   = x( 9);                             % Vap. ref. holdup condenser [kg]

% Algebraic
T1   = x(10);                               % Temperature LP evaporator [K]
T2   = x(11);                               % Temperature IP evaporator [K]
PA   = x(12);                      % Pressure inter-stage mixing node [bar]
TA   = x(13);                  % Discharge temperature first compressor [K]
TC   = x(14);                 % Discharge temperature second compressor [K]
TD   = x(15);                                   % Temperature condenser [K]
HH1  = x(16);                        % Compressor head first compressor [m]
HH2  = x(17);                       % Compressor head second compressor [m]
WV1  = x(18);                            % Vapour holdup LP evaporator [kg]
WV2  = x(19);                            % Vapour holdup IP evaporator [kg]
XV2  = x(20);                           % Valve opening, L1-constroller [-]
XV3  = x(21);                           % Valve opening, L2-constroller [-]
L1   = x(22);           % Level/volumetric liquid holdup LP evaporator [m3]
L2   = x(23);           % Level/volumetric liquid holdup IP evaporator [m3] 
L3   = x(24);                % Level/volumetric liquid holdup receiver [m3]
P1   = x(25);                                % Pressure LP evaporator [bar]
P2   = x(26);                                % Pressure IP evaporator [bar]
P3   = x(27);                                     % Pressure receiver [bar]
TP1O = x(28);         % Temperature process stream outlet LP evaporator [K]
TP2O = x(29);                 % Intermediate temperature process stream [K]
TP3O = x(30);                % Temperature cooling air outlet condenser [K]
FG1  = x(31);              % Vapour mass flowrate from LP evaporator [kg/s]
FG2  = x(32);              % Vapour mass flowrate from IP evaporator [kg/s]
FG3  = x(33);          % Vapour mass flowrate from second compressor [kg/s]
FL2  = x(34);              % Liquid mass flowrate from IP evaporator [kg/s]
FL3  = x(35);                   % Liquid mass flowrate from receiver [kg/s]
FL4  = x(36);                  % Liquid mass flowrate from condenser [kg/s]
TB   = x(37);              % Liquid mass flowrate from IP evaporator [kg/s]
PTH1 = x(38);                   % Power consumption first compressor [kJ/s]
PTH2 = x(39);                  % Power consumption second compressor [kJ/s]
Q1   = x(40);                              % Heat load LP evaporator [kJ/s]
Q2   = x(41);                              % Heat load IP evaporator [kJ/s]

% First order filters for MVs
XV2f   = x(42);
XV3f   = x(43);

% Integrated errors for level controllers
IE_L1C = x(44);
IE_L2C = x(45);

% More states (for control)
N    = x(46);
c    = x(47);
IE_c = x(48);

%% Extracting inputs and disturbances

L1sp = u( 1);                           % Level setpoint LP evaporator [m3]
L2sp = u( 2);                           % Level setpoint IP evaporator [m3]
csp  = u( 3);                                        % Compressor speed [-]
XV1  = u( 4);                                           % Valve opening [-]
FCP3 = u( 5);     % Combined mass flowrate and cp of cooling air [kJ/(s.K)] 

%% Defining parameters

W    = p.W;                               % Refrigerant inventory [kg]
V1   = p.V1;                              % Size of LP evaporator [m3]
V2   = p.V2;                              % Size of IP evaporator [m3]
V3   = p.V3;                                   % Size of receiver [m3]
VC   = p.VC;                                  % Size of condenser [m3]
C11  = p.C11;               % Coefficient for fitting compressor curve
C12  = p.C12;                                                 % --||--
C13  = p.C13;                                                 % --||--
C21  = p.C21;                                                 % --||--
C22  = p.C22;                                                 % --||--
C23  = p.C23;                                                 % --||--
C24  = p.C24;                                                 % --||--
C25  = p.C25;                                                 % --||-- 
E11  = p.E11;                                                 % --||--
E12  = p.E12;                                                 % --||--
E13  = p.E13;                                                 % --||--
E14  = p.E14;                                                 % --||--
E21  = p.E21;                                                 % --||--
E22  = p.E22;                                                 % --||--
E23  = p.E23;                                                 % --||--
E24  = p.E24;                                                 % --||--
LAM1 = p.LAM1;      % Coefficients for liq. ref. spec. vol. LP [m3/kg]
LAM2 = p.LAM2;      % Coefficients for liq. ref. spec. vol. IP [m3/kg]
LAM3 = p.LAM3;      % Coefficients for liq. ref. spec. vol. HP [m3/kg]
PHI1 = p.PHI1;            % Coefficients compressability factor LP [-]
PHI2 = p.PHI2;              % Coefficients compr. factor mix. node [-]
PHI3 = p.PHI3;     % Coefficients compressability factor condenser [-]
ZET1 = p.ZET1;  % Coefficients for vap. ref. specific enthalpy [kJ/kg]
ZET2 = p.ZET2;                                                % --||--
ZET3 = p.ZET3;                                                % --||--
DEL1 = p.DEL1;  % Coefficients for liq. ref. specific enthalpy [kJ/kg]
DEL2 = p.DEL2;                                                % --||--
DEL3 = p.DEL3;                                                % --||--
DEL4 = p.DEL4;                                                % --||--
U1A1 = p.U1A1; % Heat transfer coeff and area LP evaporator [kJ/(s.K)]
U2A2 = p.U2A2; % Heat transfer coeff and area IP evaporator [kJ/(s.K)]
U3A3 = p.U3A3;     % Heat transfer coeff and area condenser [kJ/(s.K)]
CV1  = p.CV1;                     % Valve constant XV1 [kg/(s.bar0.5)]
CV2  = p.CV2;                     % Valve constant XV2 [kg/(s.bar0.5)]
CV3  = p.CV3;                     % Valve constant XV3 [kg/(s.bar0.5)]
FCP1 = p.FCP1;      % Mass flowrate and cp of process stream [J/(s.K)] 
FCP2 = p.FCP2;      % Mass flowrate and cp of process stream [J/(s.K)] 
TP1I = T1 + p.T1diff;
TP2I = T2 + p.T2diff;            % Temperature of process stream at inlet [K]
TP3I = p.TP3I;               % Temperature of cooling air at inlet [K]
A    = p.A;                         % Coefficient for Antoine equation
B    = p.B;                                                   % --||--
C    = p.C;                                                   % --||--
R    = p.R;                                 % Gas constant [J/(mol.K)] 
MW   = p.MW;                    % Propylene molecular weight [kg/kmol] 
G    = p.G;                              % Gravity acceleration [m/s2]
C1   = p.C1;                      % Coefficient for heat cap. equation
C2   = p.C2;                                                  % --||--
C3   = p.C3;                                                  % --||--
C4   = p.C4;                                                  % --||--
C5   = p.C5;                                                  % --||--
C6   = p.C6;                                                  % --||--

%% Algebraic equations

Z1 = PHI1.a*T1^2 + PHI1.b*T1 + PHI1.c;          % Compr. factor LP evap [-]  
Z2 = PHI2.a + PHI2.b*PA/10 + PHI2.c*TB + PHI2.d*(PA/10)^2 +             ...
     PHI2.e*PA/10*TB + PHI2.f*TB^2;     % Compr. interstage mixing node [-]
Z3 = PHI3.a*TD^2 + PHI3.b*TD + PHI3.c;        % Compr. factor condenser [-]    

HS1  = HH1 / (Nf^2.19);                  % Scaled head first compressor [m]
FG1S = (Nf^1.56)*(C11*HS1 - C12)/(HS1 - C13); %Vapour vol. flow rate [m3/s]    
HS2  = HH2 / (Nf^2.11);                  % Scaled head first compressor [m]
TX   = C21 / tan((C22-HS2)/C23);                                      % [-]
FG3S = (Nf^1.79) * (C24 + C25*log(sqrt(TX^2 + 1) - TX));           % [m3/s]

HFG1 = ZET1.a*T1 + ZET1.b;            % Vap. ref. specific enthalpy [kJ/kg]
HFG2 = ZET2.a*T2 + ZET2.b;                                        % [kJ/kg]
HFG3 = ZET3.a*TC + ZET3.b;                                        % [kJ/kg] 
HFL2 = DEL2.a*T2 + DEL2.b;            % Liq. ref. specific enthalpy [kJ/kg] 
HFL3 = DEL3.a*T3 + DEL3.b;                                        % [kJ/kg]
HFL4 = DEL4.a*TD + DEL4.b;                                        % [kJ/kg]
HW1V = ZET1.a*T1 + ZET1.b;         % Vap. ref. spec. enth. LP evap. [kJ/kg]
HW1L = DEL1.a*T1 + DEL1.b;         % Liq. ref. spec. enth. LP evap. [kJ/kg]
HW2V = ZET2.a*T2 + ZET2.b;         % Vap. ref. spec. enth. IP evap. [kJ/kg]
HW2L = DEL2.a*T2 + DEL2.b;         % Liq. ref. spec. enth. IP evap. [kJ/kg]
 
A1 = exp(-U1A1/FCP1);                                                 % [-]
A2 = exp(-U2A2/FCP2);                                                 % [-]
A3 = exp(-U3A3/FCP3f);                                                % [-] 
QC = FCP3f*(TP3I - TP3O);             % Heat transfered in condenser [kJ/s]

ETA1 = E11*HS1 + E12 - 10^(E13*HS1 - E14);       % Polytrop. efficiency [-]
CPI1 = C1 + C2*T1*(C3 + C4*T1*(C5 + C6*T1));         % Heat cap [J/(mol.K)]
CPO1 = C1 + C2*TA*(C3 + C4*TA*(C5 + C6*TA));         % Heat cap [J/(mol.K)]
GAM1 = 0.5*(CPI1/(CPI1-R) + CPO1/(CPO1-R));                % Avg. gamma [-]        
POL1 = ETA1*GAM1/(GAM1-1);                                            % [-]

ETA2 = E21*HS2 + E22 - 10^(E23*HS2 - E24);       % Polytrop. efficiency [-]
CPI2 = C1 + C2*TB*(C3 + C4*TB*(C5 + C6*TB));         % Heat cap [J/(mol.K)]
CPO2 = C1 + C2*TC*(C3 + C4*TC*(C5 + C6*TC));         % Heat cap [J/(mol.K)]
GAM2 = 0.5*(CPI2/(CPI2-R) + CPO2/(CPO2-R));                % Avg. gamma [-]
POL2 = ETA2*GAM2/(GAM2-1);                                            % [-]

WL1 = W1 - WV1;                          % Liquid holdup LP evaporator [kg]
WL2 = W2 - WV2;                          % Liquid holdup IP evaporator [kg]
W3  = W - W1 - W2 - W4;                       % Liquid holdup receiver [kg]

HL1 = WL1 * HW1L;         % Enthalpy of liquid holdup in LP evaporator [kJ]
HV1 = WV1 * HW1V;         % Enthalpy of vapour holdup in LP evaporator [kJ]
HL2 = WL2 * HW2L;         % Enthalpy of liquid holdup in IP evaporator [kJ]
HV2 = WV2 * HW2V;         % Enthalpy of vapour holdup in IP evaporator [kJ]

VF1 = LAM1.a*T1^2 + LAM1.b*T1 + LAM1.c;   % Liq. ref. spec. vol. LP [m3/kg]
VF2 = LAM2.a*T2^2 + LAM2.b*T2 + LAM2.c;   % Liq. ref. spec. vol. IP [m3/kg]
VF3 = LAM3.a*TD^2 + LAM3.b*TD + LAM3.c;   % Liq. ref. spec. vol. HP [m3/kg]
                              
V1G = V1 - WL1*VF1;                          % Vapour volume in vessel [m3]
V2G = V2 - WL2*VF2;                                                % --||--
V4G = V3 + VC - W3*VF3;                                            % --||--
VG1 = V1G/WV1;                 % Vapour refrigerant specific volume [m3/kg]
VG2 = V2G/WV2;                                                     % --||--
VG4 = V4G/W4;                                                      % --||--

E_L1C   = L1sp-L1;
E_L2C   = L2sp-L2;

E_c  = csp-c;

%% Defining the derivatives
dH1     =  HFL2 * FL2 - HFG1 * FG1 + Q1;                           % [kJ/s]
dH2     =  HFL3 * FL3 - HFL2 * FL2 - HFG2 * FG2 + Q2;              % [kJ/s]
dT3     =  FL4 * (TD - T3) / W3;                                    % [K/s]
dW1     =  FL2 - FG1;                                              % [kg/s]
dW2     =  FL3 - FL2 - FG2;                                        % [kg/s]
dW4     =  FG3 - FL4;                                              % [kg/s]

dx      = [dH1; dH2; dT3; dW1; dW2; dW4];

%% Defining the residuals
T1res   =  H1 - HL1 - HV1;                                           % [kJ]
T2res   =  H2 - HL2 - HV2;                                           % [kJ]
PAres   =  FG3 - FG1 - FG2;                                        % [kg/s]
TAres   =  TA - T1 * (PA / P1)^(1 / POL1);                            % [K]
TCres   =  TC - TB * (P3 / PA)^(1 / POL2);                            % [K]
TDres   =  P3 - Z3 * R * TD / (MW * VG4 * 100);                     % [bar]
HH1res  =  HH1 - POL1 * (R * 1000/(G * MW)) * (TA - T1);              % [m]
HH2res  =  HH2 - POL2 * (R * 1000/(G * MW)) * (TC - TB);              % [m]
WV1res  =  P1 - Z1 * R * T1 / (MW * VG1 * 100);                     % [bar]
WV2res  =  P2 - Z2 * R * T2 / (MW * VG2 * 100);                     % [bar]
L1res   =  L1 - WL1 * VF1;                                           % [m3]
L2res   =  L2 - WL2 * VF2;                                           % [m3]
L3res   =  L3 - W3 * VF3;                                            % [m3]
P1res   =  P1 - exp(A - B/(T1 - C));                                % [bar]
P2res   =  P2 - exp(A - B/(T2 - C));                                % [bar]
P3res   =  P3 - exp(A - B/(TD - C));                                % [bar]
TP1Ores =  TP1O - ((1 - A1)*T1 + A1*TP1I);                            % [K]
TP2Ores =  TP2O - ((1 - A2)*T2 + A2*TP2I);                            % [K]
TP3Ores =  TP3O - ((1 - A3)*TD + A3*TP3I);                            % [K]
FG1res  =  FG1 - FG1S * MW * P1 * 100 / (T1 * R * Z1);             % [kg/s]
FG2res  =  FG2 - XV1f * CV1 * sqrt(P2 - PA);                       % [kg/s]
FG3res  =  FG3 - FG3S * MW * PA * 100 / (TB * R * Z2);             % [kg/s]
FL2res  =  FL2 - XV2f * CV2 * sqrt(P2 - P1);                       % [kg/s]
FL3res  =  FL3 - XV3f * CV3 * sqrt(P3 - P2);                       % [kg/s]
FL4res  =  FL4 + QC / (HFG3 - HFL4);                               % [kg/s]
TBres   =  TB  - (TA*FG1 + T2*FG2) / (FG1+FG2);                       % [K]
PTH1res =  PTH1 - 0.5 * FG1 * (CPO1 + CPI1) * (TA - T1) / MW;      % [kJ/s]
PTH2res =  PTH2 - 0.5 * FG3 * (CPO2 + CPI2) * (TC - TB) / MW;      % [kJ/s]
Q1res   =  Q1 - FCP1 * (TP1I - TP1O);                              % [kJ/s]
Q2res   =  Q2 - FCP2 * (TP2I - TP2O);                              % [kJ/s]

%% Controllers
% L1 controller
XV2opt = p.XV2opt;
KcL1C  = p.KcL1C;
KiL1C  = p.KiL1C;
L1Cres  = ((XV2opt - XV2) + E_L1C*KcL1C + IE_L1C*KiL1C);              % [-]

% L2 controller
XV3opt = p.XV3opt;
KcL2C  = p.KcL2C;
KiL2C  = p.KiL2C;
L2Cres  = ((XV3opt - XV3) + E_L2C*KcL2C + IE_L2C*KiL2C);              % [-]

resids = [T1res; T2res; PAres; TAres; TCres; TDres; HH1res; HH2res;     ...
          WV1res; WV2res; L1Cres; L2Cres; L1res; L2res; L3res; P1res;   ...
          P2res; P3res; TP1Ores; TP2Ores; TP3Ores; FG1res; FG2res;      ...
          FG3res; FL2res; FL3res; FL4res; TBres; PTH1res; PTH2res;      ...
          Q1res; Q2res]; 
     

%% First order low-pass filters (differential)
tau_XV2f  = p.tau_XV2f;
tau_XV3f  = p.tau_XV3f;
tau_Nf    = p.tau_Nf;
tau_XV1f  = p.tau_XV1f;
tau_FCP3f = p.tau_FCP3f;

dXV2f  = (XV2-XV2f)/tau_XV2f; 
dXV3f  = (XV3-XV3f)/tau_XV3f;
dNf    = (N-Nf)/tau_Nf;
dXV1f  = (XV1-XV1f)/tau_XV1f;
dFCP3f = (FCP3-FCP3f)/tau_FCP3f;

%% Integrated controller errors
dE_L1C  =  E_L1C;
dE_L2C  =  E_L2C;

% SOC1 controller
Nopt  = p.Nopt;
KcNC  = p.KcNC;
KiNC  = p.KiNC;

H = [373.457640163256 -4.94420157569480 -3.14236427453597;];

SOC1res = ((Nopt - N) + E_c*KcNC + IE_c*KiNC);          % [-]
cres    =  c - H(1,:)*[Nf p.alpha p.beta]';
dE_c    =  E_c;

out = [dNf; dXV1f; dFCP3f; dx; resids; dXV2f; dXV3f; dE_L1C; dE_L2C;...
    SOC1res; cres; dE_c]; 

end