function x = SS_init_x()    
    % Differential states
    H1     = 1.116625547713165e+06;                                                   % [J]
    H2     = 4.270842184367246e+05;                                                   % [J]
    T3     = 3.109509509746579e+02;                                                % [cK]
    W1     = 1.935581061716316e+03;                                                  % [kg]
    W2     = 6.371807449952997e+02;                                                  % [kg]  
    W4     = 5.285006134593906e+02;                                                   % [kg]
    x_diff = [H1; H2; T3; W1; W2; W4];

    % Algebraic states
    T1     = 2.221057033104807e+02;                                                  % [cK]
    T2     = 2.614033780005114e+02;                                                  % [cK]
    PA     = 3.838677918020478;                                                   % [bar]       
    TA     = 2.923308008937472e+02;                                                   % [bar]                    
    TC     = 3.570345554200532e+02;                                                 % [bar]
    TD     = 3.109509509746579e+02;                                                  % [bar]            
    HH1    = 7.654043882485963e+03;                                                    % [m]
    HH2    = 8.900370175149390e+03;                                                    % [m]                
    WV1    = 34.515694799935375;                                                     % [m]
    WV2    = 19.302887488995331;                                                    % [m]  
    XV2    = 0.729689969590921;
    XV3    = 0.715921408454996;
    x_alg  = [T1; T2; PA; TA; TC; TD; HH1; HH2; WV1; WV2; XV2; XV3]; % Alg.

    % Additional (non-state) variables
    L1     = 3.1;                                                    % [m3]
    L2     = 1.1;                                                  % [m3]
    L3     = 7.067077713065725;                                                  % [m3]
    P1     = 0.867901948655116;                                                   % [bar]
    P2     = 4.051824124707886;                                                   % [bar]
    P3     = 15.422953183078555;                                                  % [bar]
    TP1O   = 2.255550664871728e+02;                                                  % [cK]
    TPIM   = 2.733437983553702e+02;                                                  % [cK]
    TP3O   = 3.085770198395267e+02;                                                 % [cK]
    FG1    =  2.907863752084430;                                               % [kg/s]
    FG2    = 1.568845120257717;                                                 % [kg/s]
    FG3    = 4.476708872342146;                                                 % [kg/s]
    FL2    = 2.907863752084430;                                                 % [kg/s]
    FL3    = 4.476708872342146;                                                 % [kg/s]
    FL4    = 4.476708872342146;                                                 % [kg/s]
    TB     = 2.814924064678853e+02;                                                 % [cK]
    PTH1   =  2.822622976142466e+02;                                                   % [J]
	PTH2   =  5.449397830993895e+02;                                                   % [J]
    Q1     =  1.041604838295560e+03;                                                  % [J]
    Q2     = 74.408976969834981;

    x_add  = [L1; L2; L3; P1; P2; P3; TP1O; TPIM; TP3O; FG1; FG2; FG3; ...
              FL2; FL3; FL4; TB; PTH1; PTH2; Q1; Q2]; 

    x = [x_diff; x_alg; x_add];    % Combining diff, alg and additional
end