function [y,fval,exitflag,active_list] = SS_opt(u0,x0,p,const,socbool)
% STEADY STATE OPTIMIZATION

if (~exist('u0','var')) || isempty(u0)
    u0 = SS_init_u;
end
if (~exist('x0','var')) || isempty(x0)
    x0 = SS_init_x;
end
if (~exist('p','var')) || isempty(p)
    p = SS_init_params;
end
if (~exist('socbool','var')) || isempty(socbool)
   socbool = false;
end

g = @(x) SS_model(x,u0,p);
options = optimset('Display','none');
x0 = fsolve(g,x0,options); % Get initial guess

y0 = [u0; x0];

%% Constraints
lb=zeros(length(y0),1);
lb(1)  = 0.9;  % N
lb(2)  = 0;    % XV1
lb(3)  = 116;  % FCP3
lb(20) = 0;    % XV2
lb(21) = 0;    % XV3
lb(25) = 0;    % P1
lb(26) = 3;    % P2
lb(27) = 12;   % P3
lb(28) = 200;  % TP1O
lb(32) = 0;    % FG2
lb(34) = 0;    % FL2
lb(35) = 0;    % FL3

ub=ones(length(y0),1)*1e12;
ub(1)  = 1.1;  % N
ub(2)  = 1;    % XV1
ub(3)  = 348;  % FCP3
ub(20) = 1;    % XV2
ub(21) = 1;    % XV3
ub(25) = 2;    % P1
ub(26) = 6;    % P2
ub(27) = 18;   % P3
ub(28) = 300;  % TP1O
ub(32) = 6.31; % FG2
ub(34) = 5.47; % FL2
ub(35) = 7.18; % FL3

%% Optimization

% fmincon options
options = optimset(                                                     ...
    'ScaleProblem', 'obj-and-constr',                                   ...
    'TolFun', 10e-9,                                                    ...
    'TolCon', 10e-9,                                                    ...
    'MaxFunEvals', 1e6,                                                 ...
    'MaxIter', 1e6,                                                     ...
    'Display','none',                                                   ...
    'Algorithm','Interior-Point',                                       ...
    'Diagnostics','off',                                                ...
    'FinDiffType','central',                                            ...
    'SubproblemAlgorithm','cg');

% Call fmincon to optimize the model
if ~exist('const','var') || isempty(const)
    const = zeros(0,2);
end   
nlconpar = @(y) nlcon(y,p,const,socbool);
costpar  = @(y) cost(y,p);
[y,fval,exitflag] = fmincon(costpar,y0,[],[],[],[],lb,ub,nlconpar,options);

% Print active constraints and exitflag
active_list = SS_printactive(y,exitflag,lb,ub);

% Create process flowsheet
if length(dbstack) == 1
    SS_create_pfd(y,fval,p);
end

function j = cost(y,p)
    %% Objective function
%     %p.beta = p.beta + 1;
%     p.beta = 2;
%     p.alpha = 15;
    j = (y(38)+y(39)) + p.alpha*y(28)+p.beta*y(29);
    
function [cineq,ceq] = nlcon(y,p,const,socbool)  
%% Nonlinear inequality constraints C(y)<0
    cineq = [];
    
    %% Nonlinear equality constraints C(y)=0
    if ~exist('H','var') || isempty(H)
        load('H_matrices.mat')
    end
    if socbool
        ceq=[SS_model(y(4:end),y(1:3),p);                               ...
            y(const(:,1))-const(:,2);                                   ...
            H*y(subset)-c;                                              ...
        ];
    else
        ceq=[SS_model(y(4:end),y(1:3),p);                               ...
            y(const(:,1))-const(:,2);                                   ...
        ];
    end