function [Juu,Gy,F,y,fval] = getSensMats(u0,x0,p,const)

if (~exist('u0','var')) || isempty(u0)
    u0 = SS_init_u;
end
if (~exist('x0','var')) || isempty(x0)
    x0 = SS_init_x;
end
if (~exist('p','var')) || isempty(p)
    p = SS_init_params;
end
if ~exist('const','var') || isempty(const)
    const = zeros(0,2);
end   

%% Calculates Juu

% Get nominal solution
[y,fval] = SS_opt(u0,x0,p,const);
nom = [[1:3]',y(1:3)];

f = @(c) SS_opt(u0,x0,p,c);

% Create steps for the inputs
h = [0,0,0;nom(1,2)*0.01,0,0]';
[yf1,f1] = f(nom+h);
[yb1,b1] = f(nom-h);
[~,ce] = f(nom);

% Calculate the derivative using finite difference methods
Juu_1 = (f1-2*ce+b1)/sum(h(:))^2;

Juu = Juu_1;

%% Calculates Gy = dy/du

% Calculate the derivative using finite difference methods
GyN   = 0.5*(yf1-yb1)/sum(h(:));
Gy = GyN;

%% Calculates sensitivity matrix F

ds = {'T1diff','T2diff','TP3I','FCP1','FCP2','alpha','beta'};
F  = [];

delta  = 0.001;
for d  = ds
    f  = p.(d{1});
    pf = p; pf.(d{1}) = f+f*delta;
    yf = SS_opt(u0,x0,pf,const);
    
    pb = p; pb.(d{1}) = f-f*delta;
    yb = SS_opt(u0,x0,pb,const);
    F  = [F ((yf-yb)/(2*f*delta))];
end

F = [F;[0,0,0,0,0,1,0;...
    0,0,0,0,0,0,1]];

Gy = [Gy;0;0];

y = [y;p.alpha;p.beta];
end