function [t,x,u] = OL_integrator(c,x0,u0)
% Description

% Add path to get access to SS folder 
addpath(genpath('C:\Users\Adriaen\Dropbox\_MASTER_\MATLAB\Original'))

% Initial estimates for the states
if ~exist('c','var');
    c = zeros(0,2);
end

if ~exist('x0','var');
    x0 = SS_init_x;
end
    
if ~exist('u0','var');
    u0 = SS_init_u;
end

% Calculating the steady state
y_SS = SS_opt(u0,x0,c);

x_SS = [y_SS;y_SS(20:21);0;0];
u_SS = [3.1; 1.1; y_SS(1:3)];

%% Calculating responses to disturbances / setpoint changes

u = u_SS;
% Doing a step change in u setpoint
u(4) = u(4)*1.01;

% Additional parameters
p = SS_init_params;
%p.TP1I   = p.TP1I + 1;

p.XV2opt = y_SS(20);
p.XV3opt = y_SS(21);

% Creating a function handle to pass extra parameters to it (u and params)
g = @(t,x) OL_model(t,x,u,p);

% Defining the constant, singular mass matrix M
M = diag([ones(1,9),zeros(1,32),ones(1,4)]);

options = odeset('Mass',M,'MStateDependence','none',...
     'MassSingular','yes','RelTol',1e-6,'Vectorized','off');

tspan = 0:0.05:1000;             
[t,x] = ode15s(g,tspan,x_SS,options);

close all
plot(t,x(:,29))

tbz = (-100:0.5:-0.5)';
t = [tbz;t];
x = [(x_SS*ones(size(tbz))')';x];
u = [(u_SS*ones(size(tbz))')';(u*ones(length(t)-length(tbz),1)')'];

% Plot results
if length(dbstack)==1
	OL_plotter(t,x,x_SS,u_SS);
end


end