function p = SS_init_params()

%% Refrigerant inventory [kg] 
p.W = 6500; 

%% Vessels sizes [m3] 
p.V1 = 20;                                                  % LP evaporator 
p.V2 = 3.393;                                               % IP evaporator
p.V3 = 18.85;                                                    % Receiver 
p.VC = 4.15;                                                    % Condenser 

%% Coefficients for curve fittings for performance
% First compressor
p.C11 = 1.9928; 
p.C12 = 16791; 
p.C13 = 8756.4; 

% Second compressor
p.C21 = 0.45615; 
p.C22 = 10296; 
p.C23 = 2956.2; 
p.C24 = 0.816051; 
p.C25 = 0.27585; 

%% Coefficients for curve fittings for isentropic efficiency
% First compressor
p.E11 = 6.47E-5; 
p.E12 = 0.353; 
p.E13 = 7.101E-4; 
p.E14 = 6.5963; 

% Second compressor
p.E21 = 4.098E-5; 
p.E22 = 0.364; 
p.E23 = 1.121E-3; 
p.E24 = 12.445;

%% Coefficients for calculating specific volumes of propylene
% Low pressure
p.LAM1.a =   9.802E-9;
p.LAM1.b =  -1.053E-6;
p.LAM1.c =   0.001381;

% Intermediate pressure
p.LAM2.a =   2.223E-8;
p.LAM2.b =  -7.145E-6;
p.LAM2.c =   0.002129;

% High pressure
p.LAM3.a =   6.736E-8;
p.LAM3.b =  -3.349E-5;
p.LAM3.c =    0.00598;

%% Coefficients for calculating compressibility factors [-]
% Low pressure, first evaporator
p.PHI1.a = -1.115653785185266E-5;
p.PHI1.b =  0.003869991483971;
p.PHI1.c =  0.659220459377467;

% Intermediate pressure, mixing node
p.PHI2.a = 0.414607511696639;
p.PHI2.b =  -0.898851421515544;
p.PHI2.c = 0.004316778331929;
p.PHI2.d = -0.029680863703624;
p.PHI2.e = 0.002581535259452;
p.PHI2.f = -7.916965454791605E-6;

% High pressure, condenser
p.PHI3.a = -2.437079326095500E-5;
p.PHI3.b =  0.011121262139455;
p.PHI3.c = -0.344907274735881;

%% Enthalpies - reference enthalpy 1000kJ/kg for saturated liquid @ 273 K
p.ZET1.a = 1.26898;
p.ZET1.b = 734.198; 
p.ZET2.a = 1.41275;
p.ZET2.b = 698.814; 
p.ZET3.a = 1.75257;
p.ZET3.b = 592.804; 
p.DEL1.a = 2.18222;
p.DEL1.b = 84.2375;
p.DEL2.a = 2.40251;
p.DEL2.b = 29.8187;
p.DEL3.a = 2.81837;
p.DEL3.b = -91.3762;
p.DEL4.a = 2.81837;
p.DEL4.b = -91.3762;

%% Combined overall heat transfer coefficients and heat transfer areas 
% in the two evaporators and the condenser [J/(s.K)] 
p.U1A1 = 146.066;                                           % LP evaporator 
p.U2A2 = 5.5479;                                            % IP evaporator 
p.U3A3 = 420.643;                                               % Condenser 

%% Valve constants [kg/(s.bar0.5)]
p.CV1 = 3.39814;                                             % Vapour valve 
p.CV2 = 2.233338;                                      % First liquid valve 
p.CV3 = 1.85435;                                      % Second liquid valve

%% Combined process stream flowrate and specific heat capacity [J/(s.K)] 
p.FCP1 = 111.394;                                           % LP evaporator 
p.FCP2 = 24.32;                                             % IP evaporator 

%% Process stream inlet temperatures [K] 
p.TP1I = 235.2;                           % Process stream in LP evaporator
p.TP2I = 280.4;                           % Process stream in IP evaporator 
p.TP3I = 303.0;                              % Cooling air in the condenser 

%% Antoine Equation coefficients for propylene 
p.A = 9.0825; 
p.B = 1807.53; 
p.C = 26.15; 

%% General constants 
p.R = 8.314;                                     % Gas constant [J/(mol.K)] 
p.MW = 42.081;                       % Propylene molecular weight [kg/kmol] 
p.G = 9.81;                                   % Gravity acceleration [m/s2]

%% Specific heat capacity equation coefficients for propylene
p.C1 = 18.479424911751948; 
p.C2 = 0.007270343051175;
p.C3 = 23.438998780670026; 
p.C4 = 0.001429363603578; 
p.C5 = -11.095379346410985; 
p.C6 = 0.019414066400671; 

%% Controller settings
p.L1sp   = 3.1;
p.L2sp   = 1.1;

p.XV2opt = 0.7;
p.KcL1C  = 2.86;
p.KiL1C  = 1/37.61;

p.XV3opt = 0.7;
p.KcL2C  = 10;
p.KiL2C  = 1/37.61;

% SOC controllers
p.XV1opt = 0.5;
k     = 2.371/(0.01*0.504);
tau1  = 20;
theta = 0;
tauc  = 2; 
p.KcXV1C = (tau1/k)*(1/(tauc+theta));
p.KiXV1C = p.KcXV1C/min([tau1,4*(tauc+theta)]);

p.Nopt = 1;
k      = 234.1/(0.01*1.003);
tau1   = 10;
theta  = 0;
tauc   = 2; 
p.KcNC = (tau1/k)*(1/(tauc+theta));
p.KiNC = p.KcNC/min([tau1,4*(tauc+theta)]);

% Low pass filter time constants
p.tau_XV2f  = 0.002;
p.tau_XV3f  = 0.002;
p.tau_Nf    = 0.010;
p.tau_XV1f  = 0.002;
p.tau_FCP3f = 0.005;

end