function x = SS_init_x()    
    % Differential states
    H1     = 1.117982E6;                                                   % [J]
    H2     = 4.347168E5;                                                   % [J]
    T3     = 310.842;                                                % [cK]
    W1     = 1934.69;                                                  % [kg]
    W2     = 630.168;                                                  % [kg]  
    W4     = 526.521;                                                   % [kg]
    x_diff = [H1; H2; T3; W1; W2; W4];

    % Algebraic states
T1     = 222.486;                                                  % [cK]
    T2     = 268.474;                                                  % [cK]
    PA     = 3.84945;                                                   % [bar]       
    TA     = 291.760;                                                   % [bar]                    
    TC     = 359.871;                                                 % [bar]
    TD     = 310.842;                                                  % [bar]            
    HH1    = 7574.36;                                                    % [m]
    HH2    = 8951.01;                                                    % [m]                
    WV1    = 35.0968;                                                     % [m]
    WV2    = 23.4576;                                                    % [m]  
    XV2    = 0.66247;
    XV3    = 0.73245;
    x_alg  = [T1; T2; PA; TA; TC; TD; HH1; HH2; WV1; WV2; XV2; XV3]; % Alg.

    % Additional (non-state) variables
    L1     = 3.1;                                                    % [m3]
    L2     = 1.1;                                                  % [m3]
    L3     = 7.085;                                                  % [m3]
    P1     = 0.883654987841408;                                                   % [bar]
    P2     = 5.070104271470663;                                                   % [bar]
    P3     = 15.385558898601653;                                                  % [bar]
    TP1O   = 2.259140730949896e+02;                                                  % [cK]
    TPIM   = 2.779674765318184e+02;                                                  % [cK]
    TP3O   = 3.085006399364720e+02;                                                 % [cK]
    FG1    = 3.027199259360120;                                               % [kg/s]
    FG2    = 1.335098297645051;                                                 % [kg/s]
    FG3    = 4.362297557005171;                                                 % [kg/s]
    FL2    = 3.027199259360120;                                                 % [kg/s]
    FL3    = 4.362297557005171;                                                 % [kg/s]
    FL4    = 4.362297557005171;                                                 % [kg/s]
    TB     = 2.846334106435063e+02;                                                 % [cK]
    PTH1   = 2.897784557586991e+02;                                                   % [J]
	PTH2   = 5.325813792750275e+02;                                                   % [J]
    Q1     = 1.034396541656729e+03;                                                  % [J]
    Q2     = 59.158970746175093;

    x_add  = [L1; L2; L3; P1; P2; P3; TP1O; TPIM; TP3O; FG1; FG2; FG3; ...
              FL2; FL3; FL4; TB; PTH1; PTH2; Q1; Q2]; 

    x = [x_diff; x_alg; x_add];    % Combining diff, alg and additional
end