function Juu = getJuu()
% Calculates Juu = d2J/du2

% Initialize inputs, states and parameters
u0 = SS_init_u;
x0 = SS_init_x;
p  = SS_init_params();

% Get nominal solution
y = SS_opt(u0,x0,p);
nom = [[1:3]',y(1:3)];

f = @(c) SS_opt(u0,x0,p,c);

% Create steps for the inputs
h = [0,0,0;nom(1,2)*0.01,0,0]';
k = [0,0,0;0,nom(2,2)*0.01,0]';
    
[~,ff] = f(nom+h+k);
[~,bb] = f(nom-h-k);

[~,f1] = f(nom+h);
[~,b1] = f(nom-h);
[~,f2] = f(nom+k);
[~,b2] = f(nom-k);
[~,ce] = f(nom);

% Calculate the derivative using finite difference methods
Juu_1 = (f1-2*ce+b1)/sum(h(:))^2;
Juu_2 = (f2-2*ce+b2)/sum(k(:))^2;
Juu_d = (ff-f1-f2+2*ce-b1-b2+bb)/(sum(h(:))*sum(k(:)));

Juu = [Juu_1 Juu_d; Juu_d Juu_2];
 
end