function [Juu,Gy,F,y,fval] = getSensMats(u0,x0,p,const)

if (~exist('u0','var')) || isempty(u0)
    u0 = SS_init_u;
end
if (~exist('x0','var')) || isempty(x0)
    x0 = SS_init_x;
end
if (~exist('p','var')) || isempty(p)
    p = SS_init_params;
end
if ~exist('const','var') || isempty(const)
    const = zeros(0,2);
end   

%% Calculates Juu

% Get nominal solution
[y,fval] = SS_opt(u0,x0,p,const);
nom = [[1:3]',y(1:3)];

f = @(c) SS_opt(u0,x0,p,c);

% Create steps for the inputs
h = [0,0,0;nom(1,2)*0.01,0,0]';
k = [0,0,0;0,nom(2,2)*0.01,0]';
    
[~,ff] = f(nom+h+k);
[~,bb] = f(nom-h-k);

[yf1,f1] = f(nom+h);
[yb1,b1] = f(nom-h);
[yf2,f2] = f(nom+k);
[yb2,b2] = f(nom-k);
[~,ce] = f(nom);

% Calculate the derivative using finite difference methods
Juu_1 = (f1-2*ce+b1)/sum(h(:))^2;
Juu_2 = (f2-2*ce+b2)/sum(k(:))^2;
Juu_d = (ff-f1-f2+2*ce-b1-b2+bb)/(sum(h(:))*sum(k(:)));

Juu = [Juu_1 Juu_d; Juu_d Juu_2];

%% Calculates Gy = dy/du

% Calculate the derivative using finite difference methods
GyN   = 0.5*(yf1-yb1)/sum(h(:));
GyXV1 = 0.5*(yf2-yb2)/sum(k(:));
Gy = [GyN, GyXV1];

%% Calculates sensitivity matrix F

ds = {'TP1I','TP2I','TP3I','FCP1','FCP2'};
F  = [];

delta  = 0.001;
for d  = ds
    f  = getfield(p,d{1});
    pf = p; pf = setfield(pf,d{1},(f+f*delta));
    yf = SS_opt(u0,x0,pf,const);
    pb = p; pb = setfield(pb,d{1},(f-f*delta));
    yb = SS_opt(u0,x0,pb,const);
    F  = [F ((yf-yb)/(2*f*delta))];
end

end