function [H,c,subset,Loss,names] = optimal_measurements(n,s,l,Juu,Gy,F,y_opt)

% Function to calculate H using exact local method

if ~exist('n','var')
    n = 5;
end
if ~exist('s','var')
    s = true;
end
if ~exist('l','var')
    l = true;
end

if ~l && nargin < 7
    error('Juu, Gy, F and y_opt must be provided when l = false')
elseif l
    load('sensitivity_matrices.mat')
end
    
% Choose temperature, pressure and flow measurements, and u
subset = [1 2 6 10:15 25:37];
F  = F(subset,:);
Gy = Gy(subset,:);
Wd = diag([2,2,3,4,1]);    
Wn2 = diag([0.01,0.01,1,1,1,0.1,1,1,1,0.1,0.1,0.1,...
            1,1,1,0.1,0.1,0.1,0.1,0.1,0.1,1]);
Wn = diag(y_opt(subset)*0.01);

% Call branch  and  bound algorithm to find subset with minimal average
% loss.
addpath([pwd '/b3av'])
[~,sset] = pb3av(Gy,F,Wd,Wn,Juu,n,inf,1);

sset

names ={'N','XV1','FCP3','H1','H2','T3','W1','W2','W4','T1','T2',       ...
       'PA','TA','TC','TD','HH1','HH2','WV1','WV2','XV2','XV3','L1',    ...
       'L2','L3','P1','P2','P3','TP1O','TPIM','TP3O','FG1','FG2',   	...
       'FG3','FL2','FL3','FL4','TB','PTH1','PTH2','Q1','Q2'};
subset = subset(sset);
names = names(subset);

y_opt = y_opt(subset);
Wn_   = Wn(sset,sset);
Gy_   = Gy(sset,:);
F_    = F(sset,:);
Y_    = [F_*Wd Wn_];

H = (Gy_'/(Y_*Y_'));
c = H*y_opt;
%c2 = H(1,:)*diag(y_opt)

diffr = ((diag(Wn)-diag(Wn2))./(diag(Wn2)))'
diffr(sset)

% Calculating the average loss (formulae in pb3av is wrong)
X = Juu^(1/2)/(H*Gy_)*(H*Y_);
Loss = 1/2*norm(X,'fro')^2;

% Average loss for constant inputs
Hcon = [1 0; 0 1];
sset = [1,2];
Wncon = Wn(sset,sset);
Gycon = Gy(sset,:);
Fcon  = F(sset,:);
Ycon  = [Fcon*Wd Wncon];
X2 = Juu^(1/2)/(Hcon*Gycon)*(Hcon*Ycon);
Loss2 = 1/2*norm(X2,'fro')^2;

% Save H,c and subset for easy access 
if s
    save('H_matrices.mat','H','c','subset') 
end

end